import base64
from unittest import TestCase
from unittest.mock import Mock, patch, call

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.bi.reports import ReportsOnboardingService
from montecarlodata.queries.onboarding import TEST_TABLEAU_CRED_MUTATION, \
    TEST_LOOKER_METADATA_CRED_MUTATION, ADD_BI_CONNECTION_MUTATION, TEST_LOOKER_GIT_CLONE_CRED_MUTATION, \
    TEST_POWER_BI_CRED_MUTATION
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG

_SAMPLE_OPTIONS = {'foo': 'bar'}


class ReportOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = ReportsOnboardingService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @patch.object(ReportsOnboardingService, 'onboard')
    def test_tableau_flow(self, onboard_mock):
        self._service.onboard_tableau(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_TABLEAU_CRED_MUTATION,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            validation_response='testTableauCredentials',
            connection_response='addBiConnection',
            connection_type='tableau',
            warehouse_type='tableau',
            **_SAMPLE_BASE_OPTIONS
        )

    @patch.object(ReportsOnboardingService, 'onboard')
    def test_looker_metadata_flow(self, onboard_mock):
        self._service.onboard_looker_metadata(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_LOOKER_METADATA_CRED_MUTATION,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            validation_response='testLookerCredentials',
            connection_response='addBiConnection',
            connection_type='looker',
            warehouse_type='looker',
            **_SAMPLE_BASE_OPTIONS
        )

    @patch.object(ReportsOnboardingService, 'onboard')
    @patch('montecarlodata.integrations.onboarding.bi.reports.read_as_base64')
    def test_looker_git_flow(self, read_as_base64_mock, onboard_mock):
        file_path, service_json = 'foo', 'bar'
        base64_key = base64.b64encode(service_json.encode('utf-8'))

        input_options = {'ssh_key': file_path, **_SAMPLE_BASE_OPTIONS}
        expected_options = {**{'ssh_key': base64_key.decode()}, **_SAMPLE_BASE_OPTIONS}

        read_as_base64_mock.return_value = base64_key

        self._service.onboard_looker_git(**input_options)
        read_as_base64_mock.assert_called_once_with(file_path)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_LOOKER_GIT_CLONE_CRED_MUTATION,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            validation_response='testLookerGitCloneCredentials',
            connection_response='addBiConnection',
            connection_type='looker-git-clone',
            warehouse_type='looker',
            **expected_options
        )

    @patch.object(ReportsOnboardingService, 'onboard')
    def test_power_bi_flow(self, onboard_mock):
        self._service.onboard_power_bi(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_POWER_BI_CRED_MUTATION,
            connection_query=ADD_BI_CONNECTION_MUTATION,
            validation_response='testPowerBiCredentials',
            connection_response='addBiConnection',
            connection_type='power-bi',
            warehouse_type='power-bi',
            **_SAMPLE_BASE_OPTIONS
        )



from typing import Any, Dict, Optional
from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.hive import HiveOnboardingService
from montecarlodata.integrations.onboarding.fields import AWS_RDS_CA_CERT
from montecarlodata.queries.onboarding import TEST_DATABASE_CRED_MUTATION, \
    TEST_S3_CRED_MUTATION, TEST_HIVE_SQL_CRED_MUTATION
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class HiveOnboardingTest(TestCase):
    _DEFAULT_JOB_LIMITS = {'get_partition_locations': True, 'max_partition_locations': 50}

    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = HiveOnboardingService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock
        )

    def test_hive_mysql_flow_without_a_catalog(self):
        sample_input = {'use_ssl': True, 'catalog': None, **_SAMPLE_BASE_OPTIONS}
        self._test_hive_mysql_flow(sample_input=sample_input, job_limits=self._DEFAULT_JOB_LIMITS)

    def test_hive_mysql_flow_with_a_catalog(self):
        sample_input = {'use_ssl': True, 'catalog': 'foo', **_SAMPLE_BASE_OPTIONS}
        self._test_hive_mysql_flow(sample_input=sample_input, job_limits={'catalog_name': 'foo', **self._DEFAULT_JOB_LIMITS})

    @patch.object(HiveOnboardingService, 'onboard')
    def test_hive_s3_flow(self, onboard_mock):
        expected_options = {**_SAMPLE_BASE_OPTIONS, **{'connectionType': 'hive-s3'}}

        self._service.onboard_hive_s3(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_S3_CRED_MUTATION,
            validation_response='testS3Credentials',
            connection_type='hive-s3',
            job_types=['query_logs'],
            **expected_options
        )

    @patch.object(HiveOnboardingService, 'onboard')
    def test_hive_sql_flow(self, onboard_mock):
        self._service.onboard_hive_sql(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_HIVE_SQL_CRED_MUTATION,
            validation_response='testHiveCredentials',
            connection_type='hive',
            **_SAMPLE_BASE_OPTIONS
        )

    @patch.object(HiveOnboardingService, 'onboard')
    def _test_hive_mysql_flow(self, onboard_mock: Any, sample_input: Dict, job_limits: Optional[Dict] = None):
        expected_options = {'ssl_options': {'ca': AWS_RDS_CA_CERT}, 'connectionType': 'hive-mysql',
                            **_SAMPLE_BASE_OPTIONS}

        self._service.onboard_hive_mysql(**sample_input)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_DATABASE_CRED_MUTATION,
            validation_response='testDatabaseCredentials',
            connection_type='hive-mysql',
            job_limits=job_limits,
            **expected_options
        )



import json
import os
import pathlib
from unittest import TestCase
from unittest.mock import Mock, patch, call

import click
from box import Box
from pycarlo.core import Client

from montecarlodata.common.data import MonolithResponse
from montecarlodata.iac.mc_config_service import MonteCarloConfigService
from montecarlodata.queries.iac import CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE
from montecarlodata.utils import GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class MonteCarloConfigServiceTest(TestCase):
    @patch('os.getcwd')
    def setUp(self, getcwd) -> None:
        self._request_wrapper_mock = Mock(spec=GqlWrapper)
        self._pycarlo_client = Mock(spec=Client)
        self._print_func = Mock(spec=click.echo)
        self.project_dir = self._get_project_dir('standalone_configs')
        getcwd.return_value = self.project_dir
        self.service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock,
                                               pycarlo_client=self._pycarlo_client, print_func=self._print_func)

    def test_standalone_configs(self):
        files, template, _ = self.service.compile()
        self.assertEqual(len(files), 3)
        self.assertEqual(
            template,
            {
                'field_health': [{
                    'table': 'analytics:prod.client_hub',
                    'fields': ['account_id']
                }],
                'dimension_tracking': [{
                    'table': 'analytics:prod.customer_360',
                    'field': 'account_id'
                }]
            }
        )

    def test_embedded_dbt_configs(self):
        self.service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock,
                                               project_dir=self._get_project_dir('embedded_dbt_configs'),
                                               pycarlo_client=self._pycarlo_client)
        files, template, _ = self.service.compile()

        self.assertEqual(len(files), 4)
        self.assertEqual(
            template,
            {
                'field_health': [{
                    'table': 'analytics:prod_lineage.lineage_nodes',
                    'timestamp_field': 'created'
                }, {
                    'table': 'analytics:prod.abc'
                }, {
                    'table': 'analytics:prod.client_hub',
                    'fields': ['account_id']
                }],
                'freshness': [{
                    'table': 'analytics:prod.abc',
                    'freshness_threshold': 30,
                    'schedule': {
                        'type': 'fixed',
                        'interval_minutes': 30,
                        'start_time': '2021-07-27T19:51:00'
                    }
                }],
                'dimension_tracking': [{
                    'table': 'analytics:prod.customer_360',
                    'field': 'account_id'
                }]
            }
        )

    def test_invalid_configs(self):
        self.service = MonteCarloConfigService(_SAMPLE_CONFIG, self._request_wrapper_mock,
                                               project_dir=self._get_project_dir('invalid_configs'),
                                               pycarlo_client=self._pycarlo_client)
        self.service._abort_on_error = False

        files, template, errors_by_file = self.service.compile(abort_on_error=False)
        errors = sorted(list(errors_by_file.items()), key=lambda x: x[0])

        file, error = errors[0]
        self.assertTrue(file.endswith('dir1/dir2/monitors.yml'))
        self.assertEqual(error, ['"custom_sql" property should be a list.'])
        file, error = errors[1]
        self.assertTrue(file.endswith('dir1/monitors.yml'))
        self.assertEqual(error, ['"field_health" property should be a list.'])

    def test_apply(self):
        namespace = 'foo'
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                'response': {
                    'resourceModifications': [
                        {
                            'type': 'ResourceModificationType.UPDATE',
                            'description': 'Monitor: type=stats, table=analytics:prod.client_hub',
                            'resourceAsJson': '{"uuid": "ed4d07c3-58fd-44d0-8b2d-c1b020f45a69", "resource": null, "name": "monitor|type=stats|table=analytics:prod.client_hub|timestamp_field=<<NULL>>|where_condition=<<NULL>>", "table": "analytics:prod.customer_360", "type": "stats", "fields": [], "timestamp_field": null, "where_condition": null}'
                        }, {
                            'type': 'ResourceModificationType.UPDATE',
                            'description': 'Monitor: type=categories, table=analytics:prod.customer_360',
                            'resourceAsJson': '{"uuid": "ec3b0a80-d088-4dbe-acf5-150caf041574", "resource": null, "name": "monitor|type=categories|table=analytics:prod.customer_360|timestamp_field=<<NULL>>|where_condition=<<NULL>>|fields=account_id", "table": "analytics:prod.customer_360", "type": "categories", "fields": ["account_id"], "timestamp_field": null, "where_condition": null}'
                        }
                    ],
                    'changesApplied': True,
                    'errorsAsJson': '{}'
                }
            }
        )

        response = self.service.apply(namespace)

        config_template_as_dict = {
            'field_health': [{
                'table': 'analytics:prod.client_hub',
                'fields': ['account_id']
            }],
            'dimension_tracking': [{
                'table': 'analytics:prod.customer_360',
                'field': 'account_id'
            }]
        }

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation='createOrUpdateMonteCarloConfigTemplate',
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=False,
                resource=None
            )
        )

        self.assertEqual(response.errors, {})
        self.assertEqual(len(response.resource_modifications), 2)

    def test_apply_with_errors(self):
        namespace = 'foo'
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                'response': {
                    'resourceModifications': [],
                    'changesApplied': False,
                    'errorsAsJson': '{"validation_errors": {"monitors": {"0": {"type": ["Unknown field."]}}}}'
                }
            })

        response = self.service.apply(namespace, abort_on_error=False)

        config_template_as_dict = {
            'field_health': [{
                'table': 'analytics:prod.client_hub',
                'fields': ['account_id']
            }],
            'dimension_tracking': [{
                'table': 'analytics:prod.customer_360',
                'field': 'account_id'
            }]
        }

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation='createOrUpdateMonteCarloConfigTemplate',
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=False,
                resource=None
            )
        )

        self.assertEqual(response.errors, {'validation_errors': {'monitors': {'0': {'type': ['Unknown field.']}}}})
        self.assertEqual(len(response.resource_modifications), 0)

    def _get_project_dir(self, dir_name: str):
        return os.path.join(pathlib.Path(__file__).parent.resolve(), 'test_resources', dir_name)

    LIMIT = 2
    NAMESPACES_TABLE = """\
╒═════════════╤══════════════════════════════════╕
│ Namespace   │ Last Update Time                 │
╞═════════════╪══════════════════════════════════╡
│ namespace_1 │ 2000-01-01 00:00:00.000000+00:00 │
├─────────────┼──────────────────────────────────┤
│ namespace_2 │ 2000-01-01 00:00:00.000000+00:00 │
╘═════════════╧══════════════════════════════════╛"""

    @staticmethod
    def _mc_config_templates_response(namespace_count):
        return Box(
            {
                "get_monte_carlo_config_templates": {
                    "edges": [
                        {
                            "node": {
                                "namespace": f"namespace_{i}",
                                "last_update_time": "2000-01-01 00:00:00.000000+00:00"
                            }
                        }
                        for i in range(1, namespace_count + 1)
                    ]
                }
            }
        )

    def test_list_namespaces(self):
        self._pycarlo_client.return_value = self._mc_config_templates_response(self.LIMIT)
        self.service.list_namespaces(self.LIMIT)
        self._print_func.assert_called_once_with(self.NAMESPACES_TABLE)

    def test_list_namespaces_with_more_available(self):
        self._pycarlo_client.return_value = self._mc_config_templates_response(self.LIMIT + 1)
        self.service.list_namespaces(self.LIMIT)
        expected_calls = [
            call(self.NAMESPACES_TABLE),
            call(self.service.MORE_NS_MESSAGE),
        ]
        self._print_func.assert_has_calls(expected_calls)
